;+-----------------------------------------------------------------------------
;| Title																	: Projekt - Ivy Mainboard
;+-----------------------------------------------------------------------------
;| Funktion																	: Steuerung und berwachung der Sub-Module
;|																			: Bndelung und Verarbeitung der Sensordaten
;|																			: Kommunikation mit Umgebung
;+-----------------------------------------------------------------------------
;| Prozessor																: ATmega8
;| Takt																		: 16 MHz Quarz, 32 MHz Kerntakt, 64MHz Peripherietakt
;| Sprache																	: Assembler
;| Datum                           											: 2012-07-20
;| Version																	: 1.0
;| Autor																	: Florian Krause
;+-----------------------------------------------------------------------------
.include	"ATxmega32A4def.inc"											; Chip Definition
.include	"UserDefCVar.inc"												; User Definition
 
;-Interrupt Vector Tabelle -----------------------------------------------------
.org		0x000															; ---Reset Vector--- (programmspeicher start)
			jmp			reset												; 0x000 RESET External Pin, Power-on Reset, Brown-out Reset and Watchdog Reset
.org		0x002															; ---Oscillator Vector---
			jmp			noInt												; OSC_XOSCF_vect = 2 ; External Oscillator Failure Interrupt (NMI)
.org		0x004															; ---PortC Vector---
			jmp			noInt												; PORTC_INT0_vect = 4 ; External Interrupt 0
			jmp			noInt												; PORTC_INT1_vect = 6 ; External Interrupt 1
.org		0x008															; ---PortR Vector---
			jmp			noInt												; PORTR_INT0_vect = 8 ; External Interrupt 0
			jmp			noInt												; PORTR_INT1_vect = 10 ; External Interrupt 1
.org		0x00C															; ---DMA Vector---
			jmp			noInt												; DMA_CH0_vect = 12 ; Channel 0 Interrupt
			jmp			noInt												; DMA_CH1_vect = 14 ; Channel 1 Interrupt
			jmp			noInt												; DMA_CH2_vect = 16 ; Channel 2 Interrupt
			jmp			noInt												; DMA_CH3_vect = 18 ; Channel 3 Interrupt
.org		0x014															; ---RTC Vector---
			jmp			noInt												; RTC_OVF_vect = 20 ; Overflow Interrupt
			jmp			noInt												; RTC_COMP_vect = 22 ; Compare Interrupt
.org		0x018															; ---TWIC Vector---			
			jmp			noInt												; TWIC_TWIS_vect = 24 ; TWI Slave Interrupt
			jmp			noInt												; TWIC_TWIM_vect = 26 ; TWI Master Interrupt
.org		0x01C															; ---TCC0 Vector---			
			jmp			noInt												; TCC0_OVF_vect = 28 ; Overflow Interrupt
			jmp			noInt												; TCC0_ERR_vect = 30 ; Error Interrupt
			jmp			TCC0_CCA_ISR										; TCC0_CCA_vect = 32 ; Compare or Capture A Interrupt
			jmp			TCC0_CCB_ISR										; TCC0_CCB_vect = 34 ; Compare or Capture B Interrupt
			jmp			noInt												; TCC0_CCC_vect = 36 ; Compare or Capture C Interrupt
			jmp			noInt												; TCC0_CCD_vect = 38 ; Compare or Capture D Interrupt
.org		0x028															; ---TCC1 Vector---
			jmp			noInt												; TCC1_OVF_vect = 40 ; Overflow Interrupt
			jmp			noInt												; TCC1_ERR_vect = 42 ; Error Interrupt
			jmp			noInt												; TCC1_CCA_vect = 44 ; Compare or Capture A Interrupt
			jmp			noInt												; TCC1_CCB_vect = 46 ; Compare or Capture B Interrupt
.org		0x030															; ---SPIC Vector---			
			jmp			SPIC_ISR											; SPIC_INT_vect = 48 ; SPI Interrupt
.org		0x032															; ---USARTC0 Vector---
			jmp			noInt												; USARTC0_RXC_vect = 50 ; Reception Complete Interrupt
			jmp			noInt												; USARTC0_DRE_vect = 52 ; Data Register Empty Interrupt
			jmp			noInt												; USARTC0_TXC_vect = 54 ; Transmission Complete Interrupt
.org		0x038															; ---USARTC1 Vector---
			jmp			noInt												; USARTC1_RXC_vect = 56 ; Reception Complete Interrupt
			jmp			noInt												; USARTC1_DRE_vect = 58 ; Data Register Empty Interrupt
			jmp			noInt												; USARTC1_TXC_vect = 60 ; Transmission Complete Interrupt
.org		0x03E															; ---AES Vector---
			jmp			noInt												; AES_INT_vect = 62 ; AES Interrupt
.org		0x040															; ---NVM Vector---
			jmp			noInt												; NVM_EE_vect = 64 ; EE Interrupt
			jmp			noInt												; NVM_SPM_vect = 66 ; SPM Interrupt
.org		0x044															; ---PORTB Vector---
			jmp			noInt												; PORTB_INT0_vect = 68 ; External Interrupt 0
			jmp			noInt												; PORTB_INT1_vect = 70 ; External Interrupt 1
.org		0x056															; ---PORTE Vector---
			jmp			noInt												; PORTE_INT0_vect = 86 ; External Interrupt 0
			jmp			noInt												; PORTE_INT1_vect = 88 ; External Interrupt 1
.org		0x05A															; ---TWIE Vector---
			jmp			noInt												; TWIE_TWIS_vect = 90 ; TWI Slave Interrupt
			jmp			noInt												; TWIE_TWIM_vect = 92 ; TWI Master Interrupt
.org		0x05E															; ---TCE0 Vector---
			jmp			noInt												; TCE0_OVF_vect = 94 ; Overflow Interrupt
			jmp			noInt												; TCE0_ERR_vect = 96 ; Error Interrupt
			jmp			noInt												; TCE0_CCA_vect = 98 ; Compare or Capture A Interrupt
			jmp			noInt												; TCE0_CCB_vect = 100 ; Compare or Capture B Interrupt
			jmp			noInt												; TCE0_CCC_vect = 102 ; Compare or Capture C Interrupt
			jmp			noInt												; TCE0_CCD_vect = 104 ; Compare or Capture D Interrupt
.org		0x074															; ---USARTE0 Vector---
			jmp			USARTE0_REC_ISR										; USARTE0_RXC_vect = 116 ; Reception Complete Interrupt
			jmp			USARTE0_DRE_ISR										; USARTE0_DRE_vect = 118 ; Data Register Empty Interrupt
			jmp			noInt												; USARTE0_TXC_vect = 120 ; Transmission Complete Interrupt
.org		0x080															; ---PORTD Vector---
			jmp			noInt												; PORTD_INT0_vect = 128 ; External Interrupt 0
			jmp			noInt												; PORTD_INT1_vect = 130 ; External Interrupt 1
.org		0x084															; ---PORTA Vector---
			jmp			noInt												; PORTA_INT0_vect = 132 ; External Interrupt 0
			jmp			noInt												; PORTA_INT1_vect = 134 ; External Interrupt 1
.org		0x088															; ---ACA Vector---
			jmp			noInt												; ACA_AC0_vect = 136 ; AC0 Interrupt
			jmp			noInt												; ACA_AC1_vect = 138 ; AC1 Interrupt
			jmp			noInt												; ACA_ACW_vect = 140 ; ACW Window Mode Interrupt
.org		0x08E															; ---ADCA Vector---
			jmp			ADCA_CH0_ISR										; ADCA_CH0_vect = 142 ; Interrupt 0
			jmp			noInt												; ADCA_CH1_vect = 144 ; Interrupt 1
			jmp			noInt												; ADCA_CH2_vect = 146 ; Interrupt 2
			jmp			noInt												; ADCA_CH3_vect = 148 ; Interrupt 3
.org		0x09A															; ---TCD0 Vector---
			jmp			noInt												; TCD0_OVF_vect = 154 ; Overflow Interrupt
			jmp			noInt												; TCD0_ERR_vect = 156 ; Error Interrupt
			jmp			noInt												; TCD0_CCA_vect = 158 ; Compare or Capture A Interrupt
			jmp			noInt												; TCD0_CCB_vect = 160 ; Compare or Capture B Interrupt
			jmp			noInt												; TCD0_CCC_vect = 162 ; Compare or Capture C Interrupt
			jmp			noInt												; TCD0_CCD_vect = 164 ; Compare or Capture D Interrupt
.org		0x0A6															; ---TCD1 Vector---
			jmp			noInt												; TCD1_OVF_vect = 166 ; Overflow Interrupt
			jmp			noInt												; TCD1_ERR_vect = 168 ; Error Interrupt
			jmp			noInt												; TCD1_CCA_vect = 170 ; Compare or Capture A Interrupt
			jmp			noInt												; TCD1_CCB_vect = 172 ; Compare or Capture B Interrupt
.org		0x0AE															; ---SPID Vector---
			jmp			SPID_ISR											; SPID_INT_vect = 174 ; SPI Interrupt
.org		0x0B0															; ---USARTD0 Vector---
			jmp			noInt												; USARTD0_RXC_vect = 176 ; Reception Complete Interrupt
			jmp			noInt												; USARTD0_DRE_vect = 178 ; Data Register Empty Interrupt
			jmp			noInt												; USARTD0_TXC_vect = 180 ; Transmission Complete Interrupt
.org		0x0B6															; ---USARTD1 Vector---
			jmp			noInt												; USARTD1_RXC_vect = 182 ; Reception Complete Interrupt
			jmp			noInt												; USARTD1_DRE_vect = 184 ; Data Register Empty Interrupt
			jmp			noInt												; USARTD1_TXC_vect = 186 ; Transmission Complete Interrupt

.org		INT_VECTORS_SIZE												; ende der interrupt vectoren
			
noInt:		reti															; reti fr vestoren ohne ISR

;-System Initialisierung-------------------------------------------------------
reset:		ldi			r16,low(INTERNAL_SRAM_END)							; Initialisiere Stackpointer
			out			CPU_SPL,r16											; "
			ldi			r16,high(INTERNAL_SRAM_END)							; "
			out			CPU_SPH,r16											; "

			call		InitIntRAM											; initialisiere internen RAM

			ldi			r16,0b00000100										; setze startup flag im ram
			call		SetOPMode											; "
			
			call		MainInit											; Initialisierung der internen Peripherie

			call		SysClkConf											; Konfiguriere Systemtakt

			call		EEPROMRead											; ld remote control variablen aus eeprom

			ldi			r16,0b00001000										; setze interne int abgeschlossen flag im ram
			call		SetOPMode											; "

			sei																; global interrupt enable

			call		InitSPICSlaves										; initialisierung der SPI Slaves auf Bus C

			call		InitWLAN											; initialisiere WLAN Modul

			ldi			r16,0b00010000										; setze normal operation flag im ram
			call		SetOPMode											; 

;-Hauptprogrammschleife--------------------------------------------------------
main:		;lds			r16,CVAR_SRAM_MAIN_STATUS_2
			
			;lds			r17,CVAR_SRAM_PORTX1_OUTPUT
			;sbrc		r16,5
			;ori			r17,0b01000000
			;sbrs		r16,5
			;andi		r17,0b10111111

			;sts			CVAR_SRAM_PORTX1_OUTPUT,r17
			
			ldi			r30,low(CVAR_SRAM_ADC_RAW_I_ENGINE_LOW)
			ldi			r31,high(CVAR_SRAM_ADC_RAW_I_ENGINE_LOW)
			ldi			r28,low(CVAR_SRAM_LCD_SHADOWRAM_START)
			ldi			r29,high(CVAR_SRAM_LCD_SHADOWRAM_START)

xxx:		ld			r16,Z+
			st			Y+,r16
			ld			r16,Z+
			st			Y+,r16
			adiw		r28,1

			cpi			r30,low(CVAR_SRAM_ADC_I_ENGINE_LOW + 20)
			brne		xxx
			cpi			r31,high(CVAR_SRAM_ADC_I_ENGINE_LOW + 20)
			brne		xxx

			
			ldi			r28,low(CVAR_SRAM_LCD_SHADOWRAM_START + 128)
			ldi			r29,high(CVAR_SRAM_LCD_SHADOWRAM_START + 128)

			ldi			r16,0b11111100
			st			Y,r16
			adiw		r28,3
			st			Y,r16
			adiw		r28,3
			st			Y,r16
			adiw		r28,3
			st			Y,r16
			adiw		r28,3
			st			Y,r16
			adiw		r28,3
			st			Y,r16
			adiw		r28,3
			st			Y,r16
			adiw		r28,3
			st			Y,r16
			adiw		r28,3
			st			Y,r16
			adiw		r28,3
			st			Y,r16
			adiw		r28,3
			st			Y,r16
			adiw		r28,3
			st			Y,r16
			adiw		r28,2
			ldi			r16,0b11111111
			st			Y,r16
			adiw		r28,1
			ldi			r16,0b11111100
			st			Y,r16
			adiw		r28,3
			st			Y,r16
			adiw		r28,3
			st			Y,r16
			adiw		r28,3
			st			Y,r16
			adiw		r28,3
			st			Y,r16
			adiw		r28,3
			st			Y,r16
			adiw		r28,3
			st			Y,r16
			adiw		r28,3
			st			Y,r16
			adiw		r28,3
			st			Y,r16
			adiw		r28,3
			st			Y,r16
			adiw		r28,3
			st			Y,r16
			adiw		r28,3
			st			Y,r16
			adiw		r28,2
			ldi			r16,0b11111111
			st			Y,r16
			adiw		r28,1
			ldi			r16,0b11111100
			st			Y,r16
			adiw		r28,3
			st			Y,r16
			adiw		r28,3
			st			Y,r16
			adiw		r28,3
			st			Y,r16
			adiw		r28,3
			st			Y,r16
			adiw		r28,3
			st			Y,r16
			adiw		r28,3
			st			Y,r16
			adiw		r28,3
			st			Y,r16
			adiw		r28,3
			st			Y,r16
			adiw		r28,3
			st			Y,r16
			adiw		r28,3
			st			Y,r16
			adiw		r28,3
			st			Y,r16
			adiw		r28,2
			ldi			r16,0b11111111
			st			Y,r16
			adiw		r28,1
			
			jmp			main

;-Programmblcke---------------------------------------------------------------
.include	"ADCA_CH0_ISR.asm"
.include	"SPIC_ISR.asm"
.include	"SPID_ISR.asm"
.include	"TCC0_CCA_ISR.asm"
.include	"TCC0_CCB_ISR.asm"
.include	"USARTE0_REC_ISR.asm"
.include	"USARTE0_DRE_ISR.asm"

.include	"SysClkConf.asm"
.include	"MainInit.asm"
.include	"InitIntRAM.asm"
.include	"InitSPICSlaves.asm"
.include	"InitWLAN.asm"

.include	"DrawASCII.asm"
.include	"CalcADCValue.asm"
.include	"DoPeep.asm"
.include	"DoBlink.asm"
.include	"StatCheck.asm"
.include	"StatReact.asm"
.include	"SetOPMode.asm"
.include	"SubModControl.asm"
.include	"EEPROMRead.asm"
.include	"EEPROMWrite.asm"

;-Konstanten im Programmspeicher-----------------------------------------------
.include	"Konstanten.asm"

;-eeprom init------------------------------------------------------------------
.eseg

.db			4																	; anzahl verwendeter eeprom zellen
.db			low(550)															; batterie leer grenzwert 5,5V
.db			high(550)															; "
.db			low(600)															; batterie niedrig grenzwert 6V
.db			high(600)															; "
;------------------------------------------------------------------------------
